@@ -22,6 +22,7 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkradiobutton.h>
 #include "../gnome-print-i18n.h"
 #include "transport-selector.h"
 
@@ -31,10 +32,13 @@ static void gpa_transport_selector_destroy (GtkObject *object);
 
 static gint gpa_transport_selector_construct (GPAWidget *widget);
 
-static void gpa_ts_printer_modified (GPANode *printer, GPATransportSelector *ts);
 static void gpa_ts_rebuild_widget (GPATransportSelector *ts);
 
-static void gpa_ts_entry_changed (GtkEntry *entry, GPAWidget *gpw);
+static void gpa_ts_fentry_changed (GtkEntry *entry, GPAWidget *gpw);
+static void gpa_ts_rbentry_changed (GtkEntry *entry, GPAWidget *gpw);
+static void gpa_ts_centry_changed (GtkEntry *entry, GPAWidget *gpw);
+static void gpa_ts_rbdefault_toggled (GtkRadioButton *radiobutton, GPATransportSelector *ts);
+static void gpa_ts_rbother_toggled (GtkRadioButton *radiobutton, GPATransportSelector *ts);
 static void gpa_ts_menuitem_activate (GtkWidget *widget, gint index);
 
 static void gpa_ts_select_transport (GPATransportSelector *ts, const gchar *id);
@@ -77,7 +81,7 @@ gpa_transport_selector_class_init (GPATransportSelectorClass *klass)
 }
 
 static void
-gpa_ts_entry_changed (GtkEntry *entry, GPAWidget *gpw)
+gpa_ts_fentry_changed (GtkEntry *entry, GPAWidget *gpw)
 {
 	const guchar *text;
 
@@ -86,6 +90,40 @@ gpa_ts_entry_changed (GtkEntry *entry, GPAWidget *gpw)
 	gnome_print_config_set (gpw->config, "Settings.Transport.Backend.FileName", text);
 }
 
+static void
+gpa_ts_rbentry_changed (GtkEntry *entry, GPAWidget *gpw)
+{
+	const guchar *text;
+
+	text = gtk_entry_get_text (entry);
+
+	gnome_print_config_set (gpw->config, "Settings.Transport.Backend.Printer", text);
+}
+
+static void
+gpa_ts_centry_changed (GtkEntry *entry, GPAWidget *gpw)
+{
+	const guchar *text;
+
+	text = gtk_entry_get_text (entry);
+
+	gnome_print_config_set (gpw->config, "Settings.Transport.Backend.Command", text);
+}
+
+static void
+gpa_ts_rbdefault_toggled (GtkRadioButton *radiobutton, GPATransportSelector *ts)
+{
+
+	gtk_widget_set_sensitive (ts->rbentry, FALSE);
+
+}
+
+static void
+gpa_ts_rbother_toggled (GtkRadioButton *radiobutton, GPATransportSelector *ts)
+{
+	gtk_widget_set_sensitive (ts->rbentry, TRUE);
+}
+
 static void
 gpa_transport_selector_init (GPATransportSelector *ts)
 {
@@ -98,16 +136,34 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 	gtk_box_pack_start (GTK_BOX (ts->hbox), ts->menu, FALSE, FALSE, 0);
 
 	/* Create filename entry */
-	ts->entry = gtk_entry_new ();
-	gtk_signal_connect (GTK_OBJECT (ts->entry), "changed", GTK_SIGNAL_FUNC (gpa_ts_entry_changed), ts);
-	gtk_box_pack_start (GTK_BOX (ts->hbox), ts->entry, TRUE, TRUE, 0);
+	ts->fentry = gtk_entry_new ();
+	gtk_signal_connect (GTK_OBJECT (ts->fentry), "changed", GTK_SIGNAL_FUNC (gpa_ts_fentry_changed), ts);
+	gtk_box_pack_start (GTK_BOX (ts->hbox), ts->fentry, TRUE, TRUE, 0);
 
-	/* Create label */
-	ts->label = gtk_label_new (deflabel);
-	gtk_label_set_justify (ts->label, GTK_JUSTIFY_LEFT);
-	g_object_set (G_OBJECT (ts->label), "xalign", 0.0, NULL);
-	gtk_box_pack_end (GTK_BOX (ts->hbox), ts->label, TRUE, TRUE, 0);
-	gtk_widget_show (ts->label);
+	/* Create printname selector */
+	ts->rbhbox = gtk_hbox_new (FALSE, 4);
+	ts->rbdefault = gtk_radio_button_new_with_label (NULL, deflabel);
+	gtk_signal_connect (GTK_OBJECT (ts->rbdefault), "toggled", GTK_SIGNAL_FUNC (gpa_ts_rbdefault_toggled), ts);
+	gtk_box_pack_start (GTK_BOX (ts->rbhbox), ts->rbdefault, TRUE, TRUE, 0);
+	gtk_widget_show (ts->rbdefault);
+	ts->rbother = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (ts->rbdefault));
+	gtk_signal_connect (GTK_OBJECT (ts->rbother), "toggled", GTK_SIGNAL_FUNC (gpa_ts_rbother_toggled), ts);
+	gtk_box_pack_start (GTK_BOX (ts->rbhbox), ts->rbother, TRUE, TRUE, 0);
+	gtk_widget_show (ts->rbother);
+	
+	ts->rbentry = gtk_entry_new ();
+	gtk_widget_set_sensitive (ts->rbentry, FALSE);
+	gtk_signal_connect (GTK_OBJECT (ts->rbentry), "changed", GTK_SIGNAL_FUNC (gpa_ts_rbentry_changed), ts);
+	gtk_box_pack_end (GTK_BOX (ts->rbhbox), ts->rbentry, TRUE, TRUE, 0);
+	gtk_widget_show (ts->rbentry);
+
+	gtk_box_pack_start (GTK_BOX (ts->hbox), ts->rbhbox, TRUE, TRUE, 0);
+
+	/* Create custom command entry */
+	ts->centry = gtk_entry_new ();
+	gtk_signal_connect (GTK_OBJECT (ts->centry), "changed", GTK_SIGNAL_FUNC (gpa_ts_centry_changed), ts);
+	gtk_box_pack_end (GTK_BOX (ts->hbox), ts->centry, TRUE, TRUE, 0);
+	
 
 	ts->transportlist = NULL;
 }
@@ -146,12 +202,6 @@ gpa_transport_selector_construct (GPAWidget *gpaw)
 	gpa_ts_rebuild_widget (ts);
 }
 
-static void
-gpa_ts_printer_modified (GPANode *printer, GPATransportSelector *ts)
-{
-	gpa_ts_rebuild_widget (ts);
-}
-
 static void
 gpa_ts_rebuild_widget (GPATransportSelector *ts)
 {
@@ -177,9 +227,10 @@ gpa_ts_rebuild_widget (GPATransportSelector *ts)
 	if (!option) {
 		/* No transport node */
 		gtk_widget_hide (ts->menu);
-		gtk_widget_hide (ts->entry);
-		gtk_label_set_text (GTK_LABEL (ts->label), deflabel);
-		gtk_widget_show (ts->label);
+		gtk_widget_hide (ts->fentry);
+		gtk_widget_activate (ts->rbdefault);
+		gtk_widget_hide (ts->centry);
+		gtk_widget_show (ts->rbhbox);
 		return;
 	}
 	for (child = gpa_node_get_child (option, NULL); child != NULL; child = gpa_node_get_child (option, child)) {
@@ -188,9 +239,10 @@ gpa_ts_rebuild_widget (GPATransportSelector *ts)
 	if (!ts->transportlist) {
 		/* Empty transport list */
 		gtk_widget_hide (ts->menu);
-		gtk_widget_hide (ts->entry);
-		gtk_label_set_text (GTK_LABEL (ts->label), deflabel);
-		gtk_widget_show (ts->label);
+		gtk_widget_hide (ts->fentry);
+		gtk_widget_activate (ts->rbdefault);
+		gtk_widget_hide (ts->centry);
+		gtk_widget_show (ts->rbhbox);
 		return;
 	}
 
@@ -217,7 +269,8 @@ gpa_ts_rebuild_widget (GPATransportSelector *ts)
 			if (defid) {
 				guchar *id;
 				id = gpa_node_get_value (GPA_NODE (l->data));
-				if (id && !strcmp (id, defid)) def = idx;
+				if (id && !strcmp (id, defid))
+					def = idx;
 				g_free (id);
 			}
 			idx += 1;
@@ -227,9 +280,10 @@ gpa_ts_rebuild_widget (GPATransportSelector *ts)
 		/* No valid transports */
 		gtk_widget_destroy (menu);
 		gtk_widget_hide (ts->menu);
-		gtk_widget_hide (ts->entry);
-		gtk_label_set_text (GTK_LABEL (ts->label), deflabel);
-		gtk_widget_show (ts->label);
+		gtk_widget_hide (ts->fentry);
+		gtk_widget_activate (ts->rbdefault);
+		gtk_widget_hide (ts->centry);
+		gtk_widget_show (ts->rbhbox);
 		return;
 	}
 
@@ -282,33 +336,48 @@ gpa_ts_select_transport (GPATransportSelector *ts, const gchar *id)
 		/* Current transport is file */
 		filename = gnome_print_config_get (gpaw->config, "Settings.Transport.Backend.FileName");
 		if (filename) {
-			gtk_entry_set_text (GTK_ENTRY (ts->entry), filename);
+			gtk_entry_set_text (GTK_ENTRY (ts->fentry), filename);
 			g_free (filename);
 		} else {
-			gtk_entry_set_text (GTK_ENTRY (ts->entry), "gnome-print.out");
+			gtk_entry_set_text (GTK_ENTRY (ts->fentry), "gnome-print.out");
 		}
 		gtk_widget_show (ts->menu);
-		gtk_widget_show (ts->entry);
-		gtk_widget_hide (ts->label);
+		gtk_widget_show (ts->fentry);
+		gtk_widget_hide (ts->rbhbox);
+		gtk_widget_hide (ts->centry);
 	} else if (!strcmp (id, "lpr")) {
 		guchar *lp;
 		/* Current transport is lpr */
 		lp = gnome_print_config_get (gpaw->config, "Settings.Transport.Backend.Printer");
 		if (lp && *lp) {
-			gchar *text;
-			text = g_strdup_printf (_("Line printer: %s"), lp);
-			gtk_label_set_text (GTK_LABEL (ts->label), text);
-			g_free (text);
+			gtk_widget_activate (ts->rbother);
+			gtk_entry_set_text (GTK_ENTRY (ts->rbentry), lp);
 		} else {
-			gtk_label_set_text (GTK_LABEL (ts->label), _("Default line printer"));
+			gtk_widget_activate (ts->rbdefault);
 		}
 		if (lp) g_free (lp);
 		gtk_widget_show (ts->menu);
-		gtk_widget_hide (ts->entry);
-		gtk_widget_show (ts->label);
+		gtk_widget_hide (ts->fentry);
+		gtk_widget_show (ts->rbhbox);
+		gtk_widget_hide (ts->centry);
+	} else if (!strcmp (id, "custom")) {
+		guchar *command;
+		/* Current transport is a custom command */
+		command = gnome_print_config_get (gpaw->config, "Settings.Transport.Backend.Command");
+		if (command) {
+			gtk_entry_set_text (GTK_ENTRY (ts->centry), command);
+			g_free (command);
+		} else {
+			gtk_entry_set_text (GTK_ENTRY (ts->centry), "lpr");
+		}
+		gtk_widget_show (ts->menu);
+		gtk_widget_hide (ts->fentry);
+		gtk_widget_hide (ts->rbhbox);
+		gtk_widget_show (ts->centry);
 	} else {
 		gtk_widget_show (ts->menu);
-		gtk_widget_hide (ts->entry);
-		gtk_widget_hide (ts->label);
+		gtk_widget_hide (ts->fentry);
+		gtk_widget_hide (ts->rbhbox);
+		gtk_widget_hide (ts->centry);
 	}
 }
